# -*- coding: utf8 -*-
'''
DB MODULE - mySQL
*******************************
***  DB MODULE - mySQL
*******************************
$RCSfile: dbmod_mysql.py,v $
$Revision: 1630 $
$Author: j.holetschek $
$Date: 2021-05-26 11:07:52 +0200 (Mi, 26. Mai 2021) $

The biocase.wrapper database module to access mySQL databases.
You will need the mxODBC module to connect to a MS SQL Server DB via ODBC.
'''

from biocase.wrapper.errorclasses import DbModuleMissingError, ConnectionError
from biocase.wrapper.sql.base import sqlBase
from biocase.wrapper.dbmod.base_dbmod import DBBasemodule

import logging
log = logging.getLogger("pywrapper.dbmod.mysql")
# uses inherited ANSI SQL.

sqlBase.templates['select-limit-offset'] = 'SELECT DISTINCT #1 FROM #2 LIMIT #3 offset #4'
sqlBase.templates['count-distinct'] = 'COUNT(DISTINCT #1)'


class DBmodule(DBBasemodule):
    __dbmod__ = 'MySQL DB module using MySQLdb'

    def __init__(self, psfObj):
        # call createDBconnection method
        DBBasemodule.__init__(self, psfObj)
        # dont use parameters but integrate all parameters into SQL strings:
        self.trans.integrateParametersInSQLString()

    def createDBconnection(self):
        try:
            import MySQLdb
        except:
            raise DbModuleMissingError('MySQLdb')
        try:
            self.__class__.__version__ = '.'.join([str(i) for i in MySQLdb.version_info])
            return MySQLdb.connect(host=self.psfObj.dbIP, db=self.psfObj.database, user=self.psfObj.user, passwd=self.psfObj.password, use_unicode=True)
        except Exception as ex:
            log.exception(ex)
            raise ConnectionError

    def getTables(self):
        tables = []
        try:
            self.c = self.conn.cursor()
            self.c.execute("show tables")
            for t in self.c.fetchall():
                tables.append(str(t[0]))
        except:
            log.exception("Couldn't retrieve table info")
        finally:
            self.c.close()
        return tables

    def getColumns(self, _table):
        columns = []
        try:
            self.c = self.conn.cursor()
            self.c.execute("show columns from " + _table)
            for c in self.c.fetchall():
                columns.append(str(c[0]))
        except:
            log.error("Couldn't retrieve column info for table " + _table)
        finally:
            self.c.close()
        return columns

    def hasMetaInfo(self):
        return True
